from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import InitializeClass, DTMLFile
from Acquisition import Implicit, aq_parent, aq_base, aq_acquire
from Globals import Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item

from Products.CMFCore.utils import getToolByName

from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin, \
    IExtractionPlugin, IChallengePlugin

from zLOG import LOG, INFO

from Products.PythonScripts.PythonScript import PythonScript

import os
import random
import base64

import Liberty


# Constructor form and method
# ---------------------------

addForm = DTMLFile('dtml/addLibertyAuthPlugin', globals())


def addFunction(dispatcher, id, role, organism, generateMetadataKeys=None, REQUEST=None):
    """
    Create a new Liberty Auth Plugin.
    """
    # name with ' ' and '-' are bad for python ...
    id = id.replace(' ', '_')
    id = id.replace('-', '_')
    dest = dispatcher.Destination()
    myroot = aq_parent(dest)

    #zope_home = globals()['__builtins__']['ZOPE_HOME']
    instance_home = os.environ.get('INSTANCE_HOME')
    plugin_home = instance_home + '/Products/LibertyAuthPlugin/'

    # set liberty provider directory :
    plugin_id_filepath = plugin_home + 'liberty/' + id
    try:
        os.makedirs(plugin_id_filepath)
    except:
        pass

    root_liberty_url = myroot.absolute_url() + '/liberty/'
    metadata = plugin_id_filepath + '/' + 'metadata.xml'

    privatekey = plugin_id_filepath + '/' + 'privatekey.pem'
    publickey = plugin_id_filepath + '/' + 'publickey.pem'

    LOG('', INFO, 'generate metadata and keys : ' + str(generateMetadataKeys))

    if generateMetadataKeys:
        # set liberty provider metadata :
        Liberty.set_provider_metadata(metadata, role, root_liberty_url, organism)

        # set liberty provider private and public key :
        Liberty.set_provider_keys(privatekey, publickey)

    # set liberty plugin object :
    p = LibertyAuthPlugin(id, metadata, privatekey, publickey)
    dispatcher.Destination()._setObject(id, p)

    # set liberty urls :
    Liberty.set_liberty_urls(id, myroot)

    if REQUEST is not None:
        dispatcher.manage_main(dispatcher, REQUEST)

# Product class
# -------------

class LibertyAuthPlugin(Liberty.Liberty, BasePlugin, Implicit, Persistent, RoleManager, Item):
    """
    LibertyAuthPlugin product class.
    """

    meta_type='LibertyAuthPlugin'

    manage_options = BasePlugin.manage_options + (
        {'label' : 'Liberty Provider',
         'action' : 'editForm',
         'help' : ('LibertyAuthPlugin', 'edit.txt')
         },
        {'label' : 'Remote Providers',
         'action' : 'editLibertyProvidersForm',
         },
        {'label' : 'Liberty tokens',
         'action' : 'editLibertyTokensForm',
         'help' : ('LibertyAuthPlugin', 'edit.txt')
         },
        )

    security = ClassSecurityInfo()

    security.declareProtected('View management screens', 'editForm')
    editForm = DTMLFile('dtml/editLibertyAuthPluginForm', globals())

    security.declareProtected('View management screens', 'editLibertyProvidersForm')
    editLibertyProvidersForm = DTMLFile('dtml/editLibertyProvidersForm', globals())

    security.declareProtected('View Remote Provider screen', 'viewRemoteProvider')
    viewRemoteProviderForm = DTMLFile('dtml/viewRemoteProviderForm', globals())

    security.declareProtected('Edit Remote Provider screen', 'editRemoteProvider')
    editRemoteProviderForm = DTMLFile('dtml/editRemoteProviderForm', globals())

    security.declareProtected('Edit liberty tokens screen.', 'editLibertyTokens')
    editLibertyTokensForm = DTMLFile('dtml/editLibertyTokensForm', globals())

    def __init__(self, id, metadata, privatekey, publickey):
        self.id = id
        self._userTokens = {}
        self._emailSender = 'admin@yourploneportal'
        self._mailContent = """This is an email about user token federation.
To initiate a single sign on with token registration, go to this url :
%s"""

        Liberty.Liberty.__init__(self, metadata, privatekey, publickey)

    security.declareProtected('Add Remote Provider', 'addRemoteProvider')
    def addRemoteProvider(self, role, name, metadata, publickey, REQUEST=None):
        """
        Add a Liberty Remote Provider.
        """
        if REQUEST is not None:
            metadata_content = REQUEST.form['metadata'].read()
            publickey_content = REQUEST.form['publickey'].read()

            provider_home = self.getProviderHomePath()

            remote_provider_dirpath = provider_home + '/providers/' + name.replace(' ', '_') + \
                '-' + role + '/'
            try:
                os.makedirs(remote_provider_dirpath)
            except:
                LOG('', INFO, 'os makdirs exception ...')

            fd = open(remote_provider_dirpath + 'metadata.xml', 'w')
            fd.write(metadata_content)
            fd.close()

            fd = open(remote_provider_dirpath + 'publickey.pem', 'w')
            fd.write(publickey_content)
            fd.close()

            return self.editLibertyProvidersForm(self,
                                                 REQUEST,
                                                 manage_tabs_message = 'Remote Provider added.')

    def addUserToken(self, userId, email=None, REQUEST=None):
        """
        ...
        """
        found = False
        for ut in self._userTokens.keys():
            if self._userTokens[ut]==userId:
                return
        userToken = str(random.random()).split('.')[1]


        # send email to user :
        if email:
            portal_url = getToolByName(self, 'portal_url')
            mailhost = portal_url.MailHost
            
            register_token_url = portal_url() + '/liberty/completeUserTokenRegistration?userToken=%s' % userToken
            register_token_url_base64 = base64.encodestring(register_token_url)
            sso_url = portal_url() + '/liberty/signOn?RelayState=' + register_token_url_base64
            mMsg = self._mailContent % sso_url
            mTo = email
            mFrom = self._emailSender
            mSubj = 'User registration'

            # and send it
            mailhost.send(mMsg, mTo, mFrom, mSubj)

            LOG(INFO, '', 'url registration : ' + register_token_url)
            LOG(INFO, '', 'message : ' + mMsg)
            LOG(INFO, '', 'send token to email : ' + email)

        self._userTokens[userToken] = userId
        return self.editLibertyTokensForm(self,
                             REQUEST,
                             manage_tabs_message = 'Added user token and send email for user %s.' % userId)

    security.declareProtected('Change LibertyAuthPlugin', 'editLibertyAuthPlugin')
    def editLibertyAuthPlugin(self, REQUEST=None):
        """
        Changes Liberty Authentication Plugin preferences.
        """
        metadata_content = REQUEST.form['metadata']
        fd = open(self.getProviderHomePath() + '/metadata.xml', 'w')
        fd.write(metadata_content)
        fd.close()

        metadata_file = REQUEST.form.get('metadataFile')
        if metadata_file:
            fd = open(self.getProviderHomePath() + '/metadata.xml', 'w')
            fd.write(metadata_file.read())
            fd.close()

        privatekey_file = REQUEST.form.get('privatekeyFile')
        if privatekey_file:
            fd = open(self.getProviderHomePath() + '/privatekey.pem', 'w')
            fd.write(privatekey_file.read())
            fd.close()

        publickey_file = REQUEST.form.get('publickeyFile')
        if publickey_file:
            fd = open(self.getProviderHomePath() + '/publickey.pem', 'w')
            fd.write(publickey_file.read())
            fd.close()

        if REQUEST is not None:
            return self.editForm(self,
                                 REQUEST,
                                 manage_tabs_message = 'Liberty provider configuration changed.')

    security.declareProtected('Edit liberty tokens', 'editLibertyTokens')
    def editLibertyTokens(self, emailSender, mailContent, REQUEST=None):
        """
        Changes Liberty user token mail sending informations.
        """
        self._emailSender = emailSender
        self._mailContent = mailContent

        if REQUEST is not None:
            return self.editLibertyTokensForm(self,
                                              REQUEST,
                                              manage_tabs_message = 'Liberty mail information changed.')

    security.declareProtected('edit Remote Provider Settings', 'editRemoteProvider')
    def editRemoteProvider(self, name, metadata=None, publickey=None, REQUEST=None):
        """
        Edit remote provider metadata and public key
        """
        LOG('', INFO, ' remote provider name : ' + name)

        rprovider = None
        rproviders = self.getRemoteProviders()
        for rp in rproviders:
            if rp['name'] == name:
                rprovider = rp
                break
        if rprovider:
            if metadata:
                LOG('', INFO, ' update metadata ...')
                metadata_content = REQUEST.form['metadata'].read()
                fd = open(rp['metadata'], 'w')
                fd.write(metadata_content)
                fd.close()
            if publickey:
                LOG('', INFO, ' update public key ...')
                publickey_content = REQUEST.form['publickey'].read()
                fd = open(rp['publickey'], 'w')
                fd.write(publickey_content)
                fd.close()

        if REQUEST is not None:
            return self.editLibertyProvidersForm(self,
                                                 REQUEST,
                                                 manage_tabs_message = \
                                                 'Liberty Remote Provider changed.')

    security.declarePublic('getMetadata') 
    def getMetadata(self):
        """
        Returns provider metadata
        """
        try:
            fd = open(self._metadata)
            metadata_content = fd.read()
        except:
            metadata_content = ''
        return metadata_content

    security.declarePublic('getMetadataFilePath')
    def getMetadataFilePath(self):
        """
        Returns provider metadata file path
        """
        return self._metadata

    security.declarePublic('getMailContent')
    def getMailContent(self):
        """
        Returns mail content
        """
        return self._mailContent

    security.declarePublic('getEmailSender')
    def getEmailSender(self):
        """
        Returns email sender
        """
        return self._emailSender

    security.declarePublic('getPublicKey') 
    def getPublicKey(self):
        """
        Returns provider public key
        """
        try:
            fd = open(self._publickey)
            publickey_content = fd.read()
        except:
            publickey_content = ''
        return publickey_content

    security.declarePublic('getPublicKeyFilePath')
    def getPublicKeyFilePath(self):
        """
        Returns provider public key file path
        """
        return self._publickey

    security.declarePublic('getPrivateKeyFilePath')
    def getPrivateKeyFilePath(self):
        """
        Returns provider private key file path
        """
        return self._privatekey

    def getRemoteProviders(self):
        provider_home = self.getProviderHomePath()
        providers_filepath = []
        try:
            providers_filepath = os.listdir(provider_home + '/providers')
        except:
            LOG('', INFO, 'providers dir doesnt exists ...')

        providers = []
        for p in providers_filepath:
            name, role = p.split('-')
            metadata = provider_home + '/providers/' + p + '/metadata.xml'
            publickey = provider_home + '/providers/' + p + '/publickey.pem'
            providers.append(
                {'role' : role, 'name' : name, 'metadata' : metadata, 'publickey' : publickey}
                )
        return providers

    def getProviderHomePath(self):
        #zope_home = globals()['__builtins__']['ZOPE_HOME']
        instance_home = os.environ.get('INSTANCE_HOME')
        plugin_home = instance_home + '/Products/LibertyAuthPlugin'
        provider_home = plugin_home + '/liberty/' + self.id
        return provider_home

    def getUserIdByUserToken(self, userToken):
        for t in self._userTokens.keys():
            if userToken == t:
                return self._userTokens[t]
        return None

    def getUserTokens(self):
        tokens = []
        for t in self._userTokens.keys():
            tokens.append({'userId' : self._userTokens[t], 'userToken' : t})
        return tokens

    def isUserFederated(self, user):
        if user is None:
            return False
        LOG('', INFO, ' user : '+ str(user))
        # FIXME: Find a proper way to get userId
        nameIdentifier = self.getNameIdentifier(user.getUserId())
        if nameIdentifier:
            return True
        return False

    def manage_beforeDelete(self, item, container):
        LOG('', INFO, 'intercept delete object ...')

        # Remove remote providers settings :
        rproviders_dir = self.getProviderHomePath() + '/providers'
        try:
            rproviders = os.listdir(rproviders_dir)
        except OSError:
            rproviders = []
            rproviders_dir = None
        for rp in rproviders:
            os.remove(rproviders_dir + '/' + rp + '/metadata.xml')
            os.remove(rproviders_dir + '/' + rp + '/publickey.pem')
            os.rmdir(rproviders_dir + '/' + rp)
        if rproviders_dir:
            os.rmdir(rproviders_dir)

        # Remove provider settings :
        provider_dir = self.getProviderHomePath()
        os.remove(provider_dir + '/metadata.xml')
        os.remove(provider_dir + '/privatekey.pem')
        os.remove(provider_dir + '/publickey.pem')
        os.rmdir(provider_dir)

    def removeRemoteProvider(self, name, role, REQUEST=None):
        """
        Remove a Liberty Remote Provider.
        """
        LOG('', INFO, 'remove remote provider ' + name)

        rproviders = self.getRemoteProviders()
        rprovider = None
        for rp in rproviders:
            if rp['name'] == name:
                os.unlink(self.getProviderHomePath() + '/providers/' + name + '-' + role + \
                          '/metadata.xml')
                os.unlink(self.getProviderHomePath() + '/providers/' + name + '-' + role + \
                          '/publickey.pem')
                os.rmdir(self.getProviderHomePath() + '/providers/' + name + '-' + role)
                break

        if REQUEST is not None:
            return self.editLibertyProvidersForm(self,
                                                 REQUEST,
                                                 manage_tabs_message = 'Remote Provider removed.')

    def removeUserToken(self, userToken, REQUEST=None):
        """
        Remove user token.
        """
        for ut in self._userTokens.keys():
            if ut == userToken:
                del self._userTokens[ut]
                break
        if REQUEST is not None:
            return self.editForm(self,
                                 REQUEST,
                                 manage_tabs_message = 'User token removed.')

    security.declareProtected('View Remote Provider Settings', 'viewRemoteProvider')
    def viewRemoteProvider(self, name, REQUEST=None):
        """
        View remote provider metadata and public key
        """
        if REQUEST is not None:
            LOG('', INFO, ' remote provider name : ' + name)
            rprovider = None
            rproviders = self.getRemoteProviders()
            for rp in rproviders:
                if rp['name'] == name:
                    rprovider = rp
                    break
            if rprovider:
                fd = open(rprovider['metadata'])
                metadata = fd.read()
                fd.close()
                
                fd = open(rprovider['publickey'])
                publickey = fd.read()
                fd.close()

                return self.viewRemoteProviderForm(self,
                                                   REQUEST,
                                                   name = name,
                                                   metadata = metadata,
                                                   publickey = publickey,
                                                   manage_tabs_message = \
                                                     'Liberty Remote Provider changed.')

    # Declare available PAS interfaces :
    # ----------------------------------

    security.declarePrivate('challenge')
    def challenge(self, request, response, **kw):
        return 1

    security.declarePrivate('authenticateCredentials')
    def authenticateCredentials(self, credentials):

        nameIdentifier = credentials.get('nameIdentifier')
        if nameIdentifier is None:
            return (None, None)

        userId = self.getUserId(nameIdentifier)
        if userId is None:
            return (None, None)

        return (userId, userId)

    security.declarePrivate( 'extractCredentials' )
    def extractCredentials( self, request ):
        creds = {}
        nameIdentifier = request.SESSION.get('nameIdentifier')
        if nameIdentifier is None:
            return {}

        creds['nameIdentifier'] = nameIdentifier
        return creds


# Initialize product class
# ------------------------

classImplements(LibertyAuthPlugin, IChallengePlugin, IAuthenticationPlugin, IExtractionPlugin)
InitializeClass(LibertyAuthPlugin)
