
# $Id: WSDLObject.py,v 1.2 2006/05/09 07:12:29 fpeters Exp $
#
# Copyright (c) 2003, 2004 Dirk Datzert
# All rights reserved. Written by Dirk Datzert <dirk@datzert.de>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__version__='$Revision: 1.2 $'[11:-2]

# python imports

import os
from string import join

# global zope imports

from Globals import DTMLFile, package_home, InitializeClass

# AccessControl imports


# OFS imports

from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

# other product imports

from Products.PythonScripts.PythonScript import PythonScript

# SOAPpy package imports

from SOAPpy.wstools.WSDLTools import WSDL, \
        SoapOperationBinding, SoapBinding, SoapAddressBinding

from SOAPpy.wstools.Utility import DOM

# own package imports

_dtmldir = os.path.join( package_home( globals() ), 'dtml' )

manage_addWSDLObjectForm = \
    DTMLFile( 'manage_addWSDLObjectForm', _dtmldir )

ADD_WSDL_OBJECT_PERMISSION = 'Add WSDLObject'


class WSDLObject( SimpleItem, PropertyManager ):
    """ 
        WSDL Object

    """

    meta_type = 'WSDLObject'

    _properties = \
        ( { 'id': 'title', 'type': 'string', 'mode': 'w' },
          { 'id': 'operations', 'type': 'multiple selection', 'mode': 'w', 'select_variable': 'possible_operations' },
        )

    #################################################################
    #
    # Setting up all ZMI management screens and default login pages
    #
    #################################################################

    manage_options = PropertyManager.manage_options + \
                     SimpleItem.manage_options


    def possible_operations (self):
        """ """

        ids = []

        for dt in self.superValues(PythonScript.meta_type):
            ids.append(dt.id)

        return ids


    #################################################################
    #
    # Initialization code
    #
    #################################################################


    def __init__( self, id, title ):
        """ Create a new WSDL instance """

        self.id = id
        self.title = title
        self.operations = []


    def index_html(self, REQUEST, RESPONSE):
        """
        """

        wsdl = WSDL(targetNamespace=self.absolute_url())

        version = wsdl.version
        wsdl.namespaceDefinitions.append(('wsdl', DOM.GetWSDLUri(version)))
        wsdl.namespaceDefinitions.append(('soap', DOM.GetWSDLSoapBindingUri(version)))
        wsdl.namespaceDefinitions.append(('tns', wsdl.targetNamespace))

        soap_binding = DOM.GetWSDLSoapBindingUri(version)

        service = wsdl.addService('Service')

        for id in self.getOperations():
            portName = '%sPort' % id
            portTypeName = '%sPortType' % id
            bindingName = '%sBinding' % id
            portType = wsdl.addPortType(portTypeName)
            binding = wsdl.addBinding(bindingName, portTypeName)
            binding.addExtension(SoapBinding(soap_binding, style='rpc'))

            obj = self.aq_acquire(id, None)
            uri = obj.absolute_url()

            if obj and obj.meta_type == PythonScript.meta_type:

                body = obj.body()
                if body.find('"""') >= 0:
                    docstring = body.split('"""')[1].strip()
                else:
                    docstring = None

                parameterOrder = []
                params = obj.params().split(',')
                l = len(params)

                if l > 1:
                    for i in range(0,l):
                        parameterOrder.append(params[i].split('=')[0].strip())

                portType.addOperation(id, \
                                documentation=docstring, \
                                parameterOrder=join(parameterOrder, ' '))
                operation = binding.addOperationBinding(id)
                operation.addExtension(SoapOperationBinding(soapAction=uri, \
                                                            style='rpc'))

                port = service.addPort(portName, bindingName)
                port.addExtension(SoapAddressBinding(uri))
                
        RESPONSE.setHeader('Content-Type', 'text/xml')
        return wsdl.toXML()


    def getOperations(self):

        return self.operations


def manage_addWSDLObject( self, id, title, REQUEST=None ):
    """ Called by Zope to create and install a WSDLObject """

    wsdl = WSDLObject( id, title )

    self = self.this()
    self._setObject( id, wsdl )
 
    # return to the parent object's manage_main
    if REQUEST:
        url = REQUEST['URL1']
        if REQUEST.has_key('add_and_edit'):
            url = self[id].absolute_url()
        REQUEST.RESPONSE.redirect( '%s/manage_workspace' % url )

    else:
        wsdl = self[id]
        return wsdl 


InitializeClass( WSDL )

# vim:ts=4
